#ifndef WLR_VERSION_H
#define WLR_VERSION_H

#define WLR_VERSION_STR "0.15.1"

#define WLR_VERSION_MAJOR 0
#define WLR_VERSION_MINOR 15
#define WLR_VERSION_MICRO 1

#define WLR_VERSION_NUM ((WLR_VERSION_MAJOR << 16) | (WLR_VERSION_MINOR << 8) | WLR_VERSION_MICRO)

#endif
